﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using MvvmDataBindingDemo.Data;
using MvvmDataBindingDemo.DomainModel;

namespace MvvmDataBindingDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        List<Shipment> shipments = new List<Shipment>();

        public MainWindow()
        {
            InitializeComponent();
            LoadData();
        }

        private void LoadData()
        {
            var db = Database.Load(@"Data\database.xml");
            CountryList.ItemsSource = db.Countries;
        }

        private void CountryList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var country = CountryList.SelectedItem as Country;

            bool hasSubdivisions = (country != null) && (country.Subdivisions != null) && (country.Subdivisions.Count > 0);

            if (hasSubdivisions)
            {
                SubdivisionName.Text = country.SubdivisionName;
                SubdivisionList.IsEnabled = true;
                SubdivisionList.ItemsSource = country.Subdivisions;
            }
            else
            {
                SubdivisionName.Text = "State";
                SubdivisionList.IsEnabled = false;
                SubdivisionList.ItemsSource = null;
            }

            UpdateShipmentCost();
        }

        private void SubdivisionList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            UpdateShipmentCost();
        }

        private void UpdateShipmentCost()
        {
            var cost = GetShipmentCost();
            if (cost == null)
            {
                ShipmentCost.Text = "undefined";
                ShipmentCost.Foreground = new SolidColorBrush(Colors.Gray);
            }
            else
            {
                ShipmentCost.Text = cost.Value.ToString("0.00");
                ShipmentCost.Foreground = new SolidColorBrush(Colors.Black);
            }
        }

        private void Ship_Click(object sender, RoutedEventArgs e)
        {
            bool isValid = true;

            var subdivision = SubdivisionList.SelectedItem as Subdivision;
            if (SubdivisionList.IsEnabled && subdivision == null)
            {
                isValid = false;
                SubdivisionError.Visibility = Visibility.Visible;
                SubdivisionError.Content = "Please specify state or province";
                SubdivisionList.Focus();
            }
            else
            {
                SubdivisionError.Visibility = Visibility.Hidden;
            }

            var city = City.Text;
            if (String.IsNullOrEmpty(city))
            {
                isValid = false;
                CityError.Visibility = Visibility.Visible;
                CityError.Content = "Please enter city";
                City.Focus();
            }
            else
            {
                CityError.Visibility = Visibility.Hidden;
            }


            var country = CountryList.SelectedItem as Country;
            if (country == null)
            {
                isValid = false;
                CountryError.Visibility = Visibility.Visible;
                CountryError.Content = "Please specify country";
                CountryList.Focus();
            }
            else
            {
                CountryError.Visibility = Visibility.Hidden;
            }

            if (!isValid) return;

            var shipment = new Shipment { Country = country, Subdvision = subdivision, City = city, Cost = Decimal.Parse(ShipmentCost.Text) };
            shipments.Insert(0, shipment);

            ShipmentList.ItemsSource =
                shipments.Select(s => GetShipmentString(s));
        }

        private static string GetShipmentString(Shipment shipment)
        {
            var sb = new StringBuilder();
            sb.AppendFormat("{0:0.00}", shipment.Cost);
            sb.Append(" to ");
            sb.Append(shipment.City);
            sb.Append(", ");
            if (shipment.Subdvision != null)
            {
                sb.Append(shipment.Subdvision.Abbreviation);
                sb.Append(", ");
            }
            sb.Append(shipment.Country.Name);
            return sb.ToString();
        }

        private decimal? GetShipmentCost()
        {
            var country = CountryList.SelectedItem as Country;
            if (country == null) return null;

            var subdivision = SubdivisionList.SelectedItem as Subdivision;

            switch (country.Name)
            {
                case "USA": return GetDomesticCost(subdivision);
                case "Canada": return GetCanadianCost(subdivision);
                default: return 25.00m;
            }
        }

        private decimal? GetDomesticCost(Subdivision state)
        {
            if (state == null) return null;

            switch (state.Abbreviation)
            {
                case "AK":
                case "HI": return 14.99m;

                // North-East
                case "ME":
                case "NH":
                case "VT":
                case "MA":
                case "RI":
                case "CT":
                case "NY":
                case "NJ":
                case "PA": return 5.99m;

                // mid-West and East Coast except Florida
                case "DE":
                case "MD":
                case "VA":
                case "NC":
                case "SC":
                case "GA":
                case "OH":
                case "IL":
                case "IN":
                case "MI":
                case "WV":
                case "KY":
                case "TN":
                    return 7.99m;

                default:
                    return 9.99m;
            }
        }

        private decimal? GetCanadianCost(Subdivision province)
        {
            if (province == null) return null;
            switch (province.Abbreviation)
            {
                case "NB":
                case "QC": return 10.99m;

                default:
                    return 12.99m;
            }
        }

        private void City_GotFocus(object sender, RoutedEventArgs e)
        {
            Dispatcher.BeginInvoke(new Action(City.SelectAll));
        }
    }
}
