﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using MvvmDataBindingDemo.DomainModel;
using MvvmDataBindingDemo.Mvvm;
using ICommand=System.Windows.Input.ICommand;

namespace MvvmDataBindingDemo
{
    class MainWindowViewModel : INotifyPropertyChanged
    {
        private Model _model = new Model();

        public MainWindowViewModel()
            :
            this(new Model())
        {
        }

        internal MainWindowViewModel(Model model)
        {
            _model = model;
            ShipCommand = new DelegateCommand(OnShip);
            Shipments = new ObservableCollection<Shipment>();
        }

        public IFocusManager FocusManager {get;set;}
        public Model Model { get { return _model; } }
        public string City { get; set; }
        public ICommand ShipCommand { get; private set; }
        public ObservableCollection<Shipment> Shipments { get; private set; }

        private string _cityError;
        public string CityError
        {
            get { return _cityError; }
            set { _cityError = value; RaisePropertyChanged("CityError"); }
        }

        private Country _country;
        public Country Country
        {
            get { return _country; }
            set 
            { 
                _country = value; 
                RaisePropertyChanged("Country");
                OnCountryChanged();
            }
        }

        private string _countryError;
        public string CountryError
        {
            get { return _countryError; }
            set { _countryError = value; RaisePropertyChanged("CountryError"); }
        }

        bool _hasSubdivisions;
        public bool HasSubdivisions
        {
            get { return _hasSubdivisions; }
            set { _hasSubdivisions = value; RaisePropertyChanged("HasSubdivisions"); }
        }


        Subdivision _subdivision;
        public Subdivision Subdivision
        {
            get { return _subdivision; }
            set { 
                    _subdivision = value; 
                    RaisePropertyChanged("Subdivision"); 
                    UpdateShipmentCost(); 
                }
        }

        private string _subdivisionError;
        public string SubdivisionError
        {
            get { return _subdivisionError; }
            set { _subdivisionError= value; RaisePropertyChanged("SubdivisionError"); }
        }

        decimal? _shipmentCost;
        public decimal? ShipmentCost
        {
            get { return _shipmentCost; }
            set { _shipmentCost = value; RaisePropertyChanged("ShipmentCost"); }
        }

        private void OnCountryChanged()
        {
            HasSubdivisions = (_country != null) && 
                              (_country.Subdivisions != null) && 
                              (_country.Subdivisions.Count > 0);
            UpdateShipmentCost();
        }

        private void UpdateShipmentCost()
        {
            if (Country == null)
            {
                ShipmentCost = null;
                return;
            }

            if (HasSubdivisions && Subdivision == null)
            {
                ShipmentCost = null;
                return;
            }

            ShipmentCost = 
                Model.ShipmentCalculator.GetShipmentCost(Country, Subdivision);
        }

        private void OnShip()
        {
            if (!Validate()) return;

            var shipment = new Shipment 
            { 
                Country = Country, 
                Subdivision = Subdivision, 
                City = City, 
                Cost = Model
                        .ShipmentCalculator
                        .GetShipmentCost(Country, Subdivision)
                        .Value
            };

            Model.Send(shipment);
            Shipments.Insert(0, shipment);
        }

        private void FocusOnError(string name, string error)
        {
            if (error == null) return;
            if (FocusManager == null) return;
            FocusManager.Focus(name);
        }

        private bool Validate()
        {
            // order is important for correct focus

            SubdivisionError =
                (Country != null && HasSubdivisions && Subdivision == null) 
                    ? "Please specify " + Country.SubdivisionName 
                    : null;
            FocusOnError("SubdivisionList", SubdivisionError);

            CityError =
                (String.IsNullOrEmpty(City)) ? "Please enter city" : null;
            FocusOnError("City", CityError);

            CountryError =
                (Country == null) ? "Please specify country" : null;
            FocusOnError("CountryList", CountryError);

            return
                CityError == null && CountryError == null && SubdivisionError == null;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(name));
            }
        }
    }
}
